@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2018 by the individuals mentioned in the source code history
+#   Copyright 2007-2019 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -13,8 +13,6 @@
 #   See the License for the specific language governing permissions and
 #   limitations under the License.
 
-# TODO: Add conformance and dimname checking for weight matrix
-
 #------------------------------------------------------------------------------
 # Revision History
 #  Fri 08 Mar 2013 15:35:29 Central Standard Time -- Michael Hunter copied file from 
@@ -28,15 +26,21 @@
 
 # **DONE**
 setClass(Class = "MxFitFunctionWLS",
-	representation = representation(weights = "MxCharOrNumber"),
-	contains = "MxBaseFitFunction")
+	contains = "MxBaseFitFunction",
+	 representation = representation(
+		 type = "character",
+		 continuousType = "character",
+		 fullWeight = "logical"
+	))
 
 # **DONE**
 setMethod("initialize", "MxFitFunctionWLS",
-	function(.Object, weights, name = 'fitfunction') {
+	function(.Object, type, allContinuousMethod, fullWeight, name = 'fitfunction') {
 		.Object@name <- name
-		.Object@weights <- weights
 		.Object@vector <- FALSE
+		.Object@type <- type
+		.Object@continuousType <- allContinuousMethod
+		.Object@fullWeight <- fullWeight
 		return(.Object)
 	}
 )
@@ -45,8 +49,6 @@ setMethod("initialize", "MxFitFunctionWLS",
 setMethod("qualifyNames", signature("MxFitFunctionWLS"), 
 	function(.Object, modelname, namespace) {
 		.Object@name <- imxIdentifier(modelname, .Object@name)
-		.Object@weights <- imxConvertIdentifier(.Object@weights,
-			modelname, namespace)
 		return(.Object)
 })
 
@@ -85,9 +87,6 @@ setMethod("genericFitFunConvert", "MxFitFunctionWLS",
 		}
 		.Object@expectation <- expectIndex
 		eobj <- flatModel@expectations[[1L+expectIndex]]
-		#if(!single.na(eobj@thresholds)){
-		#	checkWLSIdentification(model, eobj)
-		#}
 		return(.Object)
 })
 
@@ -120,91 +119,18 @@ setMethod("genericFitInitialMatrix", "MxFitFunctionWLS",
 		return(matrix(as.double(NA), cols, cols))
 })
 
-setMethod("genericFitAddEntities", "MxFitFunctionWLS",
-	function(.Object, job, flatJob, labelsData) {
-		modName <- strsplit(.Object@name, split='.', fixed=TRUE)[[1]][1]
-		if(!is.null(job[[modName]]$data) && job[[modName]]$data$type %in% 'acov'){
-			W <- job[[modName]]$data$acov
-			if(any(W[row(W) != col(W)] != 0)){ # any off-diagonals are non-zero
-				job[[modName]]@fitfunction@weights <- 'WLS'
-			} else if(any(!(W %in% c(0, 1)))){ # any diagonals are non-zero non-unity
-				job[[modName]]@fitfunction@weights <- 'DLS'
-			} else {
-				job[[modName]]@fitfunction@weights <- 'ULS'
-			}
-			job@.newobjects <- TRUE
-		}
-		return(job)
-})
-
-checkWLSIdentification <- function(model, expect) {
-		#TODO check the job's expectation
-		# if it's expectation Normal
-		#	if it's identified how we want, then do nothing
-		#	else add constraints and change expectation
-		#		might need to re-do expectationFunctionAddEntities and/or flattening
-		# else if it's RAM
-		#	if it's identified how we want, then do nothing
-		#	else throw error
-		# else if it's LISREL
-		#	if it's identified how we want, then do nothing
-		#	else throw error
-		# else throw error that WLS doesn't work for that yet.
-
-	type <- strsplit(is(expect)[1], "MxExpectation")[[1]][2]
-	if(type=='RAM'){
-		A <- expect@A
-		S <- expect@S
-		F <- expect@F
-		M <- expect@M
-		# ???
-		# mxComputeOnce the RAM expectation?
-		stop("Ordinal WLS with the RAM expectation is not yet implemented.  For now, use mxExpectationNormal.")
-	}
-	else if(type=='LISREL'){
-		# ???
-		# mxComputeOnce the LISREL expectation?
-		stop("Ordinal WLS with the LISREL expectation is not yet implemented.  For now, use mxExpectationNormal.")
-	}
-	else if(type=='Normal'){
-		#Note: below strategy does NOT catch cases where starting values
-		# make the criteria true by accident.
-		theCov <- mxEvalByName(expect@covariance, model, compute=TRUE) #Perhaps save time with cache?
-		covDiagsOne <- all( ( (diag(theCov) - 1)^2 ) < 1e-7 )
-		meanName <- expect@means
-		if(!single.na(meanName)){
-			theMeans <- mxEvalByName(expect@means, model, compute=TRUE)
-			meanZeroNA <- all( ( (diag(theMeans) - 0)^2 ) < 1e-7 )
-		} else {
-			meanZeroNA <- TRUE
-		}
-		if(!covDiagsOne || !meanZeroNA){
-			stop("Model not identified in the way required by WLS.")
-		}
-	}
-	else{
-		stop(paste("The model", omxQuotes(model@name), "has an expectation",
-			"that is not RAM, LISREL, or Normal.  WLS does not know what to do."))
-	}
-}
-
 # **DONE**
-mxFitFunctionWLS <- function(weights="ULS") {
-	if (is.na(weights)) weights <- as.character(NA)
-	if (!(weights %in% c("ULS", "DWLS", "WLS"))){
-		stop("weights must be either 'ULS', 'DLS' or 'WLS'")
-		}
-	return(new("MxFitFunctionWLS", weights))
+mxFitFunctionWLS <- function(type=c('WLS','DWLS','ULS'),
+			     allContinuousMethod=c("cumulants", "marginals"),
+			     fullWeight=TRUE) {
+	type <- match.arg(type)
+	allContinuousMethod <- match.arg(allContinuousMethod)
+	return(new("MxFitFunctionWLS", type, allContinuousMethod, as.logical(fullWeight)))
 }
 
 # **DONE**
 displayMxFitFunctionWLS <- function(fitfunction) {
 	cat("MxFitFunctionWLS", omxQuotes(fitfunction@name), '\n')
-	if (single.na(fitfunction@weights)) {
-		cat("$weights : NA \n")
-	} else {
-		cat("$weights :", omxQuotes(fitfunction@weights), '\n')
-	}
 	if (length(fitfunction@result) == 0) {
 		cat("$result: (not yet computed) ")
 	} else {
@@ -227,7 +153,6 @@ setMethod("show", "MxFitFunctionWLS", function(object) {
 imxWlsStandardErrors <- function(model){
 	#TODO add safety check
 	# Is it a WLS fit function
-	# Does it have data of type=='acov'
 	# Does the data have @fullWeight
 	isMultiGroupModel <- is.null(model$expectation) && (class(model$fitfunction) %in% "MxFitFunctionMultigroup")
 	fwMsg <- "Terribly sorry, master, but you cannot compute standard errors without the full weight matrix."
@@ -337,7 +262,7 @@ imxWlsChiSquare <- function(model, J=NA){
 	if(prod(dim(jacOC)) > 0){
 		x2 <- t(e) %*% jacOC %*% ginv( as.matrix(t(jacOC) %*% W %*% jacOC) ) %*% t(jacOC) %*% e
 	} else {x2 <- 0}
-	df <- qr(jacOC)$rank - numOrdinal*2 #subtract the number of ordinal means and variances
+	df <- qr(jacOC)$rank
 	
 	dvd <- try(solve( t(jac) %*% V %*% jac ), silent=TRUE)
 	if(class(dvd) != "try-error"){
@@ -363,10 +288,9 @@ imxWlsChiSquare <- function(model, J=NA){
 approveWLSIntervals <- function(flatModel, modelName) {
 	ff <- flatModel@fitfunctions[[ paste0(modelName, '.fitfunction') ]]
 	if (is(ff, "MxFitFunctionWLS")) {
-		ds <- flatModel@datasets[[ paste0(modelName, '.data') ]]
-		if (any(abs(ds$acov - ds$fullWeight) > 1e-6)) {
+		if (ff@type != 'WLS') {
 			stop(paste("Confidence intervals are not supported for DWLS or ULS. ",
-				"Try mxSE or switch", omxQuotes(ds$name), "to full WLS"))
+				"Try mxSE or switch", omxQuotes(modelName), "to full WLS"))
 		}
 	} else if (is(ff, "MxFitFunctionMultigroup")) {
 		for (g1 in flatModel@fitfunction$groups) {
