@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2018 by the individuals mentioned in the source code history
+#   Copyright 2007-2019 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -293,10 +293,8 @@ addEntriesLISREL <- function(model, entries){
 		data <- data[[1]]
 		model@data <- data
 		# If the data are WLS, then change the fit function to WLS away from the default ML.
-		if(model@data@type=="acov" && class(model@fitfunction) %in% "MxFitFunctionML"){
+		if(is(model@data,"MxDataLegacyWLS") && class(model@fitfunction) %in% "MxFitFunctionML"){
 			model[['fitfunction']] <- mxFitFunctionWLS()
-		} else if(model@data@type %in% c('raw', 'cov') && !(class(model@fitfunction) %in% "MxFitFunctionML")){
-			model[['fitfunction']] <- mxFitFunctionML()
 		}
 		#model[['F']] <- createMatrixF(model) #TODO something here about re-structuring LX and LY if needed
 	}
