@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2018 by the individuals mentioned in the source code history
+#   Copyright 2007-2019 by the individuals mentioned in the source code history
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -13,7 +13,7 @@
 #   See the License for the specific language governing permissions and
 #   limitations under the License.
 
-gitVersion <- "v2.11.5"
+gitVersion <- "v2.12.1"
 
 mxVersion <- function (model=NULL, verbose=TRUE) {
 	pvers <- pkg_globals$myVersion
@@ -24,7 +24,7 @@ mxVersion <- function (model=NULL, verbose=TRUE) {
 		if ("Darwin" ==Sys.info()["sysname"]){
 			msg = paste(msg, "\nMacOS:", system("sw_vers -productVersion", intern=TRUE))
 		}
-		msg = paste(msg, "\nDefault optimiser: ", mxOption(NULL, "Default optimizer"), sep="")
+		msg = paste(msg, "\nDefault optimizer: ", mxOption(NULL, "Default optimizer"), sep="")
 		msg = paste(msg, "\nNPSOL-enabled?: ", ifelse(imxHasNPSOL(),"Yes","No"), sep="")
 		msg = paste(msg, "\nOpenMP-enabled?: ", ifelse(imxHasOpenMP(),"Yes","No"), sep="")
 		if (!is.null(model)) {
